/*
 * @Author: wyb
 * @Description: 基于axios的请求封装
 * @FilePath: /httpClient/httpClient.js
 */

import axios from 'axios'
import Cookies from 'js-cookie'

let pending = []
let CancelToken = axios.CancelToken
let cancelPending = (config) => {
  pending.forEach((item, index) => {
    if(config) {
      if(item.UrlPath === config.url && item.isAllowCancel && config.isAllowCancel && item.axiosModel === config.axiosModel) {
        item.Cancel() // 取消请求
        pending.splice(index, 1) // 移除当前请求
      }
    }
  })
}

class HttpClient {
  /**
   *
   * @param { Object } options
   * @param { Object } config axios配置
   */
  constructor(options, config) {
    this.router = options && options.router
    // 请求队列
    this.requestQueue = new Map()
    this.env = options && options.env || 'production'
    this.errorHandle = options && options.errorHandle || function(message) {}
    this.config = config || {}
    this.failAuth = options && options.failAuth || function() {}
    this.failRequest = options && options.failRequest || function(res) {}
  }
  /**
   *@description 对外暴露请求方法
   * @param { Object } options 自定义配置项
   */
  request(options = {}) {
    const unionOptions = Object.assign(this.getInsideConfig(), this.config, options)
    const instance = axios.create(unionOptions)
    this.interceptors(instance)
    return instance
  }

  //默认配置
  getInsideConfig() {
    let config = {
      headers: {
        Authorization:  sessionStorage.getItem('Authorization') || Cookies.get('Token') || ''
      }
    }
    if(document.cookie) {
      config.headers['ip'] = Cookies.get('ip') || 'http://127.0.0.1'
    }
    return config
  }

  // 销毁队列
  destroy(url) {
    this.requestQueue.delete[url]
  }
  // 成功处理
  successResponse(data) {
    if(data.invalidToken) {
      this.destroy()
    }
  }
  // 统一拦截
  interceptors(instance) {
    instance.interceptors.request.use(config => {
      cancelPending(config)
      config.cancelToken = new CancelToken(res => {
        pending.push({
          UrlPath: config.url,
          Cancel: res,
          isAllowCancel: config.isAllowCancel || false,
          axiosModel: config.axiosModel || {}
        })
      })
      return config
    }, error => Promise.reject(error))
    instance.interceptors.response.use(res => {
      const { data, status } = res
      this.successResponse(data)
      cancelPending(res.config)
      if(data.success) {
        return data
      } else {
        // 错误信息传给调用者 （api返回数据格式应保持一致）
        this.errorHandle(data.error)
        return Promise.reject(data)
      }
      return { data, status }
    }, error => {
      if(error.toString() === 'Cancel') return Promise.reject(error)
      if(error.response.status === 401) {
        this.failAuth()
      } else {
        this.failRequest(error.response)
      }
      console.error(error.response)
      return Promise.reject(error)
    })
  }
}

export default HttpClient